@@ -2,7 +2,7 @@
 
 /**
   * SquirrelMail Lockout Plugin
-  * Copyright (c) 2004-2007 Paul Lesniewski <paul@squirrelmail.org>
+  * Copyright (c) 2004-2008 Paul Lesniewski <paul@squirrelmail.org>
   * Licensed under the GNU GPL. For full terms see the file COPYING.
   *
   * @package plugins
@@ -170,6 +170,7 @@ function activate_captcha_plugin_do($args)
    //
    if (check_sm_version(1, 5, 2))
    {
+/* more recently, 1.5.2 fixed this issue......
       include_once(SM_PATH . 'functions/strings.php');
       include_once(SM_PATH . 'functions/prefs.php');
       $prefs_backend = do_hook('prefs_backend', $null);
@@ -180,6 +181,7 @@ function activate_captcha_plugin_do($args)
       } else {
           include_once(SM_PATH . 'functions/file_prefs.php');
       }
+*/
    }
    else
       include_once(SM_PATH . 'functions/prefs.php');
@@ -539,10 +541,24 @@ function check_lockout_do($args)
   * Redirect current page request to another page,
   * or the built in (fake) login error page.
   *
+  * We also log lockout rules violation events here.
+  *
   */
 function lockout_redirect($uri)
 {
 
+   // log this event - note that to use this,
+   // you'll need to have the Squirrel Logger plugin
+   // installed and activated and you'll have to add
+   // a "LOCKOUT" event type to its configuration
+   //
+   global $log_violated_lockout_rules, $login_username;
+   if ($log_violated_lockout_rules && is_plugin_enabled('squirrel_logger'))
+   {
+      include_once(SM_PATH . 'plugins/squirrel_logger/functions.php');
+      sl_logit('LOCKOUT', sprintf('User "%s" has tripped lockout rules; login disallowed', $login_username));
+   }
+
    sqsession_destroy();
 
    if (strpos($uri, 'http') === 0)
@@ -766,16 +782,36 @@ function check_failed_login_count_do($args)
 
          // send alert to admin if necessary
          //
-         global $domain;
+         global $domain, $log_violated_max_user_logins;
          sq_change_text_domain('lockout');
          if ($number_minutes > 0)
          {
             if (!$lockout_window) 
             {
+               // log this event - note that to use this, you'll need to have
+               // the Squirrel Logger plugin installed and activated and you'll
+               // have to add a "LOCKOUT" event type to its configuration
+               //
+               if ($log_violated_max_user_logins && is_plugin_enabled('squirrel_logger'))
+               {
+                  include_once(SM_PATH . 'plugins/squirrel_logger/functions.php');
+                  sl_logit('LOCKOUT', sprintf('User "%s" (domain "%s") has attempted (and failed) to log in %d times in the last %d minutes; %s has been LOCKED OUT PERMANENTLY', $login_username, $domain, sizeof($new_times), $number_minutes, $login_username));
+               }
+
                l_report_abuse(sprintf(_("NOTICE: User \"%s\" (domain \"%s\") has attempted (and failed) to log in %d times in the last %d minutes.\n\nTimes:\n%s\n\n%s has been LOCKED OUT PERMANENTLY.\n\nTo unlock this user, remove the \"%s_TOO_MANY_FAILED_LOGIN_ATTEMPTS\" setting from the \"lockout_plugin_login_failure_information\" preference set."), $login_username, $domain, sizeof($new_times), $number_minutes, $failed_times, $login_username, $login_username), sprintf(_(" --- LOCKED OUT - %s"), $login_username));
             }
             else
             {
+               // log this event - note that to use this, you'll need to have
+               // the Squirrel Logger plugin installed and activated and you'll
+               // have to add a "LOCKOUT" event type to its configuration
+               //
+               if ($log_violated_max_user_logins && is_plugin_enabled('squirrel_logger'))
+               {
+                  include_once(SM_PATH . 'plugins/squirrel_logger/functions.php');
+                  sl_logit('LOCKOUT', sprintf('User "%s" (domain "%s") has attempted (and failed) to log in %d times in the last %d minutes; %s has been LOCKED OUT for %d minutes', $login_username, $domain, sizeof($new_times), $number_minutes, $login_username, $lockout_window));
+               }
+
                l_report_abuse(sprintf(_("NOTICE: User \"%s\" (domain \"%s\") has attempted (and failed) to log in %d times in the last %d minutes.\n\nTimes:\n%s\n\n%s has been LOCKED OUT for %d minutes.\n\nTo unlock this user before then, remove the \"%s_TOO_MANY_FAILED_LOGIN_ATTEMPTS\" setting from the \"lockout_plugin_login_failure_information\" preference set."), $login_username, $domain, sizeof($new_times), $number_minutes, $failed_times, $login_username, $lockout_window, $login_username), sprintf(_(" --- LOCKED OUT - %s"), $login_username));
             }
          }
@@ -783,10 +819,30 @@ function check_failed_login_count_do($args)
          {
             if (!$lockout_window) 
             {
+               // log this event - note that to use this, you'll need to have
+               // the Squirrel Logger plugin installed and activated and you'll
+               // have to add a "LOCKOUT" event type to its configuration
+               //
+               if ($log_violated_max_user_logins && is_plugin_enabled('squirrel_logger'))
+               {
+                  include_once(SM_PATH . 'plugins/squirrel_logger/functions.php');
+                  sl_logit('LOCKOUT', sprintf('User "%s" (domain "%s") has attempted (and failed) to log in %d times; %s has been LOCKED OUT PERMANENTLY', $login_username, $domain, sizeof($new_times), $login_username));
+               }
+
                l_report_abuse(sprintf(_("NOTICE: User \"%s\" (domain \"%s\") has attempted (and failed) to log in %d times.\n\nTimes:\n%s\n\n%s has been LOCKED OUT PERMANENTLY.\n\nTo unlock this user, remove the \"%s_TOO_MANY_FAILED_LOGIN_ATTEMPTS\" setting from the \"lockout_plugin_login_failure_information\" preference set."), $login_username, $domain, sizeof($new_times), $failed_times, $login_username, $login_username), sprintf(_(" --- LOCKED OUT - %s"), $login_username));
             }
             else
             {
+               // log this event - note that to use this, you'll need to have
+               // the Squirrel Logger plugin installed and activated and you'll
+               // have to add a "LOCKOUT" event type to its configuration
+               //
+               if ($log_violated_max_user_logins && is_plugin_enabled('squirrel_logger'))
+               {
+                  include_once(SM_PATH . 'plugins/squirrel_logger/functions.php');
+                  sl_logit('LOCKOUT', sprintf('User "%s" (domain "%s") has attempted (and failed) to log in %d times; %s has been LOCKED OUT for %d minutes', $login_username, $domain, sizeof($new_times), $login_username, $lockout_window));
+               }
+
                l_report_abuse(sprintf(_("NOTICE: User \"%s\" (domain \"%s\") has attempted (and failed) to log in %d times.\n\nTimes:\n%s\n\n%s has been LOCKED OUT for %d minutes.\n\nTo unlock this user before then, remove the \"%s_TOO_MANY_FAILED_LOGIN_ATTEMPTS\" setting from the \"lockout_plugin_login_failure_information\" preference set."), $login_username, $domain, sizeof($new_times), $failed_times, $login_username, $lockout_window, $login_username), sprintf(_(" --- LOCKED OUT - %s"), $login_username));
             }
          }
@@ -859,16 +915,36 @@ function check_failed_login_count_do($args)
 
          // send alert to admin if necessary
          //
-         global $domain;
+         global $domain, $log_violated_max_IP_logins;
          sq_change_text_domain('lockout');
          if ($number_minutes > 0)
          {
             if (!$lockout_window) 
             {
+               // log this event - note that to use this, you'll need to have
+               // the Squirrel Logger plugin installed and activated and you'll
+               // have to add a "LOCKOUT" event type to its configuration
+               //
+               if ($log_violated_max_IP_logins && is_plugin_enabled('squirrel_logger'))
+               {
+                  include_once(SM_PATH . 'plugins/squirrel_logger/functions.php');
+                  sl_logit('LOCKOUT', sprintf('Someone at %s has attempted (and failed) to log in %d times in the last %d minutes; %s has been LOCKED OUT PERMANENTLY', $remote_addr, sizeof($new_times), $number_minutes, $remote_addr));
+               }
+
                l_report_abuse(sprintf(_("NOTICE: Someone at %s has attempted (and failed) to log in %d times in the last %d minutes.\n\nTimes:\n%s\n\n%s has been LOCKED OUT PERMANENTLY.\n\nTo unlock this client address, remove the \"%s_TOO_MANY_FAILED_LOGIN_ATTEMPTS\" setting from the \"lockout_plugin_login_failure_information\" preference set."), $remote_addr, sizeof($new_times), $number_minutes, $failed_times, $remote_addr, $remote_addr), sprintf(_(" --- LOCKED OUT - %s"), $remote_addr));
             }
             else
             {
+               // log this event - note that to use this, you'll need to have
+               // the Squirrel Logger plugin installed and activated and you'll
+               // have to add a "LOCKOUT" event type to its configuration
+               //
+               if ($log_violated_max_IP_logins && is_plugin_enabled('squirrel_logger'))
+               {
+                  include_once(SM_PATH . 'plugins/squirrel_logger/functions.php');
+                  sl_logit('LOCKOUT', sprintf('Someone at %s has attempted (and failed) to log in %d times in the last %d minutes; %s has been LOCKED OUT for %d minutes', $remote_addr, sizeof($new_times), $number_minutes, $remote_addr, $lockout_window));
+               }
+
                l_report_abuse(sprintf(_("NOTICE: Someone at %s has attempted (and failed) to log in %d times in the last %d minutes.\n\nTimes:\n%s\n\n%s has been LOCKED OUT for %d minutes.\n\nTo unlock this client address, remove the \"%s_TOO_MANY_FAILED_LOGIN_ATTEMPTS\" setting from the \"lockout_plugin_login_failure_information\" preference set."), $remote_addr, sizeof($new_times), $number_minutes, $failed_times, $remote_addr, $lockout_window, $remote_addr), sprintf(_(" --- LOCKED OUT - %s"), $remote_addr));
             }
          }
@@ -876,10 +952,30 @@ function check_failed_login_count_do($args)
          {
             if (!$lockout_window) 
             {
+               // log this event - note that to use this, you'll need to have
+               // the Squirrel Logger plugin installed and activated and you'll
+               // have to add a "LOCKOUT" event type to its configuration
+               //
+               if ($log_violated_max_IP_logins && is_plugin_enabled('squirrel_logger'))
+               {
+                  include_once(SM_PATH . 'plugins/squirrel_logger/functions.php');
+                  sl_logit('LOCKOUT', sprintf('Someone at %s has attempted (and failed) to log in %d times; %s has been LOCKED OUT PERMANENTLY', $remote_addr, sizeof($new_times), $remote_addr));
+               }
+
                l_report_abuse(sprintf(_("NOTICE: Someone at %s has attempted (and failed) to log in %d times.\n\nTimes:\n%s\n\n%s has been LOCKED OUT PERMANENTLY.\n\nTo unlock this client address, remove the \"%s_TOO_MANY_FAILED_LOGIN_ATTEMPTS\" setting from the \"lockout_plugin_login_failure_information\" preference set."), $remote_addr, sizeof($new_times), $failed_times, $remote_addr, $remote_addr), sprintf(_(" --- LOCKED OUT - %s"), $remote_addr));
             }
             else
             {
+               // log this event - note that to use this, you'll need to have
+               // the Squirrel Logger plugin installed and activated and you'll
+               // have to add a "LOCKOUT" event type to its configuration
+               //
+               if ($log_violated_max_IP_logins && is_plugin_enabled('squirrel_logger'))
+               {
+                  include_once(SM_PATH . 'plugins/squirrel_logger/functions.php');
+                  sl_logit('LOCKOUT', sprintf('Someone at %s has attempted (and failed) to log in %d times; %s has been LOCKED OUT for %d minutes', $remote_addr, sizeof($new_times), $remote_addr, $lockout_window));
+               }
+
                l_report_abuse(sprintf(_("NOTICE: Someone at %s has attempted (and failed) to log in %d times.\n\nTimes:\n%s\n\n%s has been LOCKED OUT for %d minutes.\n\nTo unlock this client address before then, remove the \"%s_TOO_MANY_FAILED_LOGIN_ATTEMPTS\" setting from the \"lockout_plugin_login_failure_information\" preference set."), $remote_addr, sizeof($new_times), $failed_times, $remote_addr, $lockout_window, $remote_addr), sprintf(_(" --- LOCKED OUT - %s"), $remote_addr));
             }
          }
@@ -946,18 +1042,38 @@ function check_failed_login_count_do($args)
             $failed_times .= date('H:i:s Y-m-d', $time) . "\n";
 
 
-         // send alert to admin if necessary
+         // log event and send alert to admin if necessary
          //
-         global $domain;
+         global $domain, $log_CAPTCHA_enabled;
          sq_change_text_domain('lockout');
          if ($number_minutes > 0)
          {
             if (!$lockout_window) 
             {
+               // log this event - note that to use this, you'll need to have
+               // the Squirrel Logger plugin installed and activated and you'll
+               // have to add a "LOCKOUT" event type to its configuration
+               //
+               if ($log_CAPTCHA_enabled && is_plugin_enabled('squirrel_logger'))
+               {
+                  include_once(SM_PATH . 'plugins/squirrel_logger/functions.php');
+                  sl_logit('LOCKOUT', sprintf('Someone at %s has attempted (and failed) to log in %d times in the last %d minutes; users attempting to log in from %s are PERMANENTLY required to enter a CAPTCHA code when logging in', $remote_addr, sizeof($new_times), $number_minutes, $remote_addr));
+               }
+
                l_report_abuse(sprintf(_("NOTICE: Someone at %s has attempted (and failed) to log in %d times in the last %d minutes.\n\nTimes:\n%s\n\nUsers attempting to log in from %s are PERMANENTLY required to enter a CAPTCHA code when logging in.\n\nTo remove the CAPTCHA requirement, remove the \"%s_CAPTCHA_TOO_MANY_FAILED_LOGIN_ATTEMPTS\" setting from the \"lockout_plugin_login_failure_information\" preference set."), $remote_addr, sizeof($new_times), $number_minutes, $failed_times, $remote_addr, $remote_addr), sprintf(_(" --- LOCKED OUT - %s"), $remote_addr));
             }
             else
             {
+               // log this event - note that to use this, you'll need to have
+               // the Squirrel Logger plugin installed and activated and you'll
+               // have to add a "LOCKOUT" event type to its configuration
+               //
+               if ($log_CAPTCHA_enabled && is_plugin_enabled('squirrel_logger'))
+               {
+                  include_once(SM_PATH . 'plugins/squirrel_logger/functions.php');
+                  sl_logit('LOCKOUT', sprintf('Someone at %s has attempted (and failed) to log in %d times in the last %d minutes; users attempting to log in from %s will be required to enter a CAPTCHA code when logging in for the next %d minutes', $remote_addr, sizeof($new_times), $number_minutes, $remote_addr, $lockout_window));
+               }
+
                l_report_abuse(sprintf(_("NOTICE: Someone at %s has attempted (and failed) to log in %d times in the last %d minutes.\n\nTimes:\n%s\n\nUsers attempting to log in from %s will be required to enter a CAPTCHA code when logging in for the next %d minutes.\n\nTo remove the CAPTCHA requirement, remove the \"%s_CAPTCHA_TOO_MANY_FAILED_LOGIN_ATTEMPTS\" setting from the \"lockout_plugin_login_failure_information\" preference set."), $remote_addr, sizeof($new_times), $number_minutes, $failed_times, $remote_addr, $lockout_window, $remote_addr), sprintf(_(" --- LOCKED OUT - %s"), $remote_addr));
             }
          }
@@ -965,10 +1081,30 @@ function check_failed_login_count_do($args)
          {
             if (!$lockout_window) 
             {
+               // log this event - note that to use this, you'll need to have
+               // the Squirrel Logger plugin installed and activated and you'll
+               // have to add a "LOCKOUT" event type to its configuration
+               //
+               if ($log_CAPTCHA_enabled && is_plugin_enabled('squirrel_logger'))
+               {
+                  include_once(SM_PATH . 'plugins/squirrel_logger/functions.php');
+                  sl_logit('LOCKOUT', sprintf('Someone at %s has attempted (and failed) to log in %d times; users attempting to log in from %s are PERMANENTLY required to enter a CAPTCHA code when logging in', $remote_addr, sizeof($new_times), $remote_addr));
+               }
+
                l_report_abuse(sprintf(_("NOTICE: Someone at %s has attempted (and failed) to log in %d times.\n\nTimes:\n%s\n\nUsers attempting to log in from %s are PERMANENTLY required to enter a CAPTCHA code when logging in.\n\nTo remove the CAPTCHA requirement, remove the \"%s_CAPTCHA_TOO_MANY_FAILED_LOGIN_ATTEMPTS\" setting from the \"lockout_plugin_login_failure_information\" preference set."), $remote_addr, sizeof($new_times), $failed_times, $remote_addr, $remote_addr), sprintf(_(" --- LOCKED OUT - %s"), $remote_addr));
             }
             else
             {
+               // log this event - note that to use this, you'll need to have
+               // the Squirrel Logger plugin installed and activated and you'll
+               // have to add a "LOCKOUT" event type to its configuration
+               //
+               if ($log_CAPTCHA_enabled && is_plugin_enabled('squirrel_logger'))
+               {
+                  include_once(SM_PATH . 'plugins/squirrel_logger/functions.php');
+                  sl_logit('LOCKOUT', sprintf('Someone at %s has attempted (and failed) to log in %d times; users attempting to log in from %s will be required to enter a CAPTCHA code when logging in for the next %d minutes', $remote_addr, sizeof($new_times), $remote_addr, $lockout_window));
+               }
+
                l_report_abuse(sprintf(_("NOTICE: Someone at %s has attempted (and failed) to log in %d times.\n\nTimes:\n%s\n\nUsers attempting to log in from %s will be required to enter a CAPTCHA code when logging in for the next %d minutes.\n\nTo remove the CAPTCHA requirement before then, remove the \"%s_CAPTCHA_TOO_MANY_FAILED_LOGIN_ATTEMPTS\" setting from the \"lockout_plugin_login_failure_information\" preference set."), $remote_addr, sizeof($new_times), $failed_times, $remote_addr, $lockout_window, $remote_addr), sprintf(_(" --- LOCKED OUT - %s"), $remote_addr));
             }
          }
@@ -999,86 +1135,34 @@ function l_report_abuse($error, $title='')
    if (empty($lockout_notification_addresses))
       return;
 
-   include_once(SM_PATH . 'functions/mime.php');
-   include_once(SM_PATH . 'class/mime/Message.class.php'); // only needed for STABLE
-   include_once(SM_PATH . 'class/mime/Rfc822Header.class.php'); // only needed for STABLE
-   include_once(SM_PATH . 'class/mime/ContentType.class.php'); // only needed for STABLE
-   include_once(SM_PATH . 'class/mime/AddressStructure.class.php'); // only needed for STABLE
 
    sq_change_text_domain('lockout');
 
+
    $user = substr($login_username, 0, strpos($login_username, $at));
 
+
    $body = $error . "\n\n" . _("User account:") . ' ' . $login_username
          . "\n" . _("Domain:") . ' ' . $domain
          . "\n" . _("Timestamp: ") . date('Y-m-d H:i:s');
 
-   $report = new Message();
-   $header  = new Rfc822Header();
-
-   $report->setBody($body);
-   $content_type = new ContentType('text/plain');
-   global $special_encoding, $default_charset;
-   if ($special_encoding)
-      $rfc822_header->encoding = $special_encoding;
-   else
-      $rfc822_header->encoding = '8bit';
-   if ($default_charset)
-      $content_type->properties['charset']=$default_charset;
-   $header->content_type = $content_type;
-
-   $header->parseField('To', $lockout_notification_addresses);
-   $header->parseField('From', 'noreply');
-   $header->parseField('Subject', _("[POSSIBLE BRUTEFORCE ABUSE]")
-                     . (empty($title) ? '' : ': ' . $title)
-                     . ' (' . $login_username . ')');
-   $report->rfc822_header = $header;
-//sm_print_r($report);exit;
-
-
-
-   global $useSendmail;
-
-
-
-   // ripped from compose.php (1.4.4)
-   //
-    if (!$useSendmail) {
-        require_once(SM_PATH . 'class/deliver/Deliver_SMTP.class.php');
-        $deliver = new Deliver_SMTP();
-        global $smtpServerAddress, $smtpPort, $pop_before_smtp, $smtp_auth_mech;
-
-        $authPop = (isset($pop_before_smtp) && $pop_before_smtp) ? true : false;
-
-        if ($smtp_auth_mech == 'none' && !$authPop) {
-                $user = '';
-                $pass = '';
-        } else {
-                global $key, $onetimepad;
-                $user = $login_username;
-                $pass = OneTimePadDecrypt($key, $onetimepad);
-        }
-
-        $stream = $deliver->initStream($report,$domain,0,
-                $smtpServerAddress, $smtpPort, $user, $pass, $authPop);
-    } else {
-        require_once(SM_PATH . 'class/deliver/Deliver_SendMail.class.php');
-        global $sendmail_path;
-        $deliver = new Deliver_SendMail();
-        $stream = $deliver->initStream($report,$sendmail_path);
-    }
-
-
-    $succes = false;
-    if ($stream) {
-        $length = $deliver->mail($report, $stream);
-        $succes = $deliver->finalizeStream($stream);
-    }
 
+   $subject = _("[POSSIBLE BRUTEFORCE ABUSE]")
+            . (empty($title) ? '' : ': ' . $title)
+            . ' (' . $login_username . ')';
 
 
    sq_change_text_domain('squirrelmail');
 
+
+   // function found in SM core 1.5.2+ and Compatibility plugin 2.0.11+
+   // (suppress include error, since file is only needed for 1.5.2+,
+   // otherwise Compatibility has us covered)
+   //
+   @include_once(SM_PATH . 'functions/compose.php');
+   $success = sq_send_mail($lockout_notification_addresses, $subject, $body, 'noreply@' . $domain);
+
+
 }
 
 
